@@ -1,28 +1,58 @@
 from string import ascii_lowercase as al, ascii_uppercase as au, digits as d
+from random import choices, seed as set_seed
 from PIL import Image, ImageDraw, ImageFont
+from typing import List, Tuple
 from tabulate import tabulate
-from random import choices
 
 # Important: Always put out new version on PyPI before pushing to github
 
+DEFAULT = "(default)"
 
-class PasswordCard:
+
+class PasswordCard(object):
     """
     Generates a Password Card
 
     Functions:
       get_password: gets the password for [Keyword]
       save: creates an image containing the table
+      raw: returns the raw card data
+
+    Variables:
+      version: version
+      __version__: version
+
+    Magic Functions:
+      __init__: initialises the password card
+      __getitem__: getitem (card[row, column])
+      __str__: converts the card data to a more readable format
     """
-    def __init__(self, keyword_length: int, segment_length: int = 3):
+    __version__, version = ["1.2.1"] * 2
+
+    def __init__(self,
+                 keyword_length: int,
+                 segment_length: int = 3,
+                 seed=None):
         """
         :param keyword_length: has to be the length of the keyword
+        :param segment_length: segment length
+        :param seed: seed for random.seed
         """
-        self.card, printable = [["ABC", "DEF", "GHI", "JKL", "MNO", "PQR", "STU", "VWX", "ZY", "."]], al + au + d + "{%}?!.,_;\\'[]#"
-        for i in range(keyword_length):
-            self.card.append([''.join(choices(printable, k=segment_length + i - i)) for i in range(10)])  # + i - i because "unused variable"
+        self.card, printable = [["ABC", "DEF", "GHI", "JKL", "MNO", "PQR", "STU", "VWX", "ZY",
+                                 "."]], al + au + d + "{%}?!.,_;\\'[]#"
 
-    def get_password(self, keyword: str):
+        # Setting seed
+        if not callable(seed):
+            set_seed(seed)
+        else:
+            raise TypeError(f"Expected seed to be of type not callable; but got {type(seed).__name__}")
+
+        # Creating card
+        for i in range(keyword_length):
+            self.card.append([''.join(choices(printable, k=segment_length + i - i)) for i in
+                              range(10)])  # + i - i because "unused variable"
+
+    def get_password(self, keyword: str) -> str:
         """
         Gets the password for [keyword]
         """
@@ -45,7 +75,11 @@ class PasswordCard:
 
         return password
 
-    def save(self, filename: str, font: str = f"{__file__.replace('__init__.py', '')}CONSOLA.TTF", font_size: int = 15, txt: bool = False):
+    def save(self,
+             filename: str,
+             font: str = DEFAULT,
+             font_size: int = 15,
+             txt: bool = False):
         """
         Saves the card as a png or txt
 
@@ -57,23 +91,21 @@ class PasswordCard:
         text = self.__str__()
 
         if txt:
-            with open(filename, "w") as file:
+            with open(filename, "w", errors="ignore") as file:
                 file.write(text)
             return
 
-        try:
-            # load font
-            font = ImageFont.truetype(font, font_size)
-        except OSError:
-            print(font)
-            print(str(__file__))
+        # load font
+        if font == DEFAULT:
+            font = f"{__file__.replace('__init__.py', '')}CONSOLA.TTF"
+        font = ImageFont.truetype(font, font_size)
 
         # determine the size that the image should have
         size, lines = {"x": 0, "y": 0}, []
         for line in text.split("\n"):
-            lines.append(font.getsize(line)[0]+20)
+            lines.append(font.getsize(line)[0] + 20)
         size["x"] = max(lines)
-        size["y"] = len(text.split("\n"))*font.getsize(text)[1]+20
+        size["y"] = int(len(text.split("\n")) * font.getsize(text)[1] * 1.067)
 
         # create image
         img = Image.new("RGB", tuple(size.values()), color="white")
@@ -97,6 +129,33 @@ class PasswordCard:
             filename = f"card_{randint(1000, 100000000)}.png"
         """
 
+    def raw(self) -> List[List[str]]:
+        return self.card
+
+    def __getitem__(self, row_column: Tuple[int or str, int]) -> str:
+        row, column = row_column
+        if not isinstance(row, int):
+            if str(row) in self.card[0]:
+                row = self.card[0].index(str(row))
+            else:
+                raise ValueError("Invalid row!")
+        else:
+            row -= 1
+        if row > len(self.card[0]) - 1:
+            raise IndexError("Row to big!")
+
+        if not isinstance(column, int):
+            raise IndexError("Column has to be of type int!")
+        elif column > len(self.card) - 1:
+            raise IndexError("Column to big!")
+
+        if column <= 0:
+            raise IndexError("Column to small!")
+        if row < 0:
+            raise IndexError("Row to small!")
+
+        return self.card[column][row]
+
     def __str__(self) -> str:
         tabulated_card = tabulate(
             self.card,
@@ -111,10 +170,10 @@ class PasswordCard:
         # number looks a bit weird (normally '| 1 |...', bug: '|1 |')
         card, i = '\n'.join(tabulated_card.split("\n")[:2]) + '\n', 1
         for line in tabulated_card.split("\n")[2:]:
-            if len(str(i)) > len(str(i-1)):
-                card += line.replace(f" {i-1}", str(i)) + '\n'
+            if len(str(i)) > len(str(i - 1)):
+                card += line.replace(f" {i - 1}", str(i)) + '\n'
             else:
-                card += line.replace(str(i-1), str(i)) + '\n'
+                card += line.replace(str(i - 1), str(i)) + '\n'
             i += 1
 
         return card
